"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logger = void 0;
const setting_1 = require("./setting");
const types_1 = require("./types");
const colors_1 = require("colors");
const dayjs_1 = __importDefault(require("dayjs"));
const getTimeFormated = () => `[${dayjs_1.default().format('YYYY-MM-DD HH:mm:ss.SSS')}] `;
const getLoggerTimeSegment = () => (setting_1.Settings.loggerTime ? getTimeFormated() : '');
const stringify = (args) => args.map((v) => {
    var _a;
    return !(v instanceof Error) && Number(v === null || v === void 0 ? void 0 : v.length) > setting_1.Settings.loggerFoldToLenLimit
        ? setting_1.Settings.loggerInfoStringify
            ? `\n${colors_1.cyan(`[${((_a = v.constructor) === null || _a === void 0 ? void 0 : _a.name) || typeof v} length: ${v.length}]`)}\n${String(v)}\n`
            : v.length
        : v;
});
const getTarget = (target) => typeof target === 'function' ? target() : target;
exports.logger = {
    error(level, target, protocolOrTraceId, ...args) {
        let protocol = undefined;
        let traceId = undefined;
        if (protocolOrTraceId instanceof types_1.ProtocolBase) {
            protocol = protocolOrTraceId;
            traceId = protocol === null || protocol === void 0 ? void 0 : protocol.traceId;
        }
        else if (protocolOrTraceId)
            traceId = protocolOrTraceId;
        if (level > setting_1.ErrorLevel.off &&
            level <= setting_1.Settings.errorLevel &&
            setting_1.Settings.errorFilter(getTarget(target), protocol)) {
            target = getTarget(target);
            console.error(`${getLoggerTimeSegment()}Error occurred(${setting_1.ErrorLevel[level]}) ${(protocol === null || protocol === void 0 ? void 0 : protocol.protocol) || ''} ${(protocol === null || protocol === void 0 ? void 0 : protocol.addr) || ''} (${traceId}):`);
            if (target)
                console.error('target:', target);
            console.error(...stringify(args));
            console.error('\n');
        }
    },
    log(level, target, protocolOrTraceId, ...args) {
        let protocol = undefined;
        let traceId = undefined;
        if (protocolOrTraceId instanceof types_1.ProtocolBase) {
            protocol = protocolOrTraceId;
            traceId = protocol === null || protocol === void 0 ? void 0 : protocol.traceId;
        }
        else if (protocolOrTraceId)
            traceId = protocolOrTraceId;
        if (level > setting_1.LogLevel.off &&
            level <= setting_1.Settings.logLevel &&
            setting_1.Settings.logFilter(getTarget(target), protocol)) {
            target = getTarget(target);
            console.log(`${getLoggerTimeSegment()}Log(${setting_1.LogLevel[level]}) ${(protocol === null || protocol === void 0 ? void 0 : protocol.protocol) || ''} ${(protocol === null || protocol === void 0 ? void 0 : protocol.addr) || ''} (${traceId}):`);
            if (target)
                console.error('target:', target);
            console.log(...stringify(args));
            console.log('\n');
        }
    },
    logVital(...args) {
        console.log(getLoggerTimeSegment(), ...args);
    },
    doseLog: () => setting_1.Settings.logLevel !== setting_1.LogLevel.off || setting_1.Settings.errorLevel !== setting_1.ErrorLevel.off,
};
//# sourceMappingURL=logger.js.map