"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.raceConnect = void 0;
const logger_1 = require("./logger");
const net_1 = __importDefault(require("net"));
const channelDiagnostic_1 = require("../stats/channelDiagnostic");
const setting_1 = require("./setting");
const types_1 = require("./types");
/**
 * 通过比较第一个包的响应时间选取最快的路线
 * @param connectData 当请求的方法是CONNECT时需传递
 */
const raceConnect = (targets, protocol) => {
    const haveProxy = !!targets.find((v) => !v.target.notProxy);
    let finished = false;
    let maxRecvCount = 0;
    const dataCache = [];
    let msockPair = null;
    let connectedSocks = [];
    const cbMap = {};
    let connectCb = () => {
        var _a;
        (_a = cbMap['connect']) === null || _a === void 0 ? void 0 : _a.call(cbMap);
        connectCb = null;
    };
    const raceRecvDataMap = new Map();
    let minRacingCost = Infinity;
    let minRecvSockPair = null;
    let minCancelCb = null;
    let judgeTimeOut = -Infinity;
    const judgeWin = (isTimeout = true, winCb) => {
        var _a;
        if (msockPair)
            return;
        const win = isTimeout || socks.filter((v) => v).length === 1;
        if (win && minRecvSockPair) {
            msockPair = minRecvSockPair;
            minRecvSockPair = null;
            minCancelCb = null;
            socks.forEach((v) => v !== (msockPair === null || msockPair === void 0 ? void 0 : msockPair.sock) && (v === null || v === void 0 ? void 0 : v.destroy()));
            ['data', 'end', 'error'].forEach((ev) => msockPair === null || msockPair === void 0 ? void 0 : msockPair.sock.on(ev, ev === 'error'
                ? (err) => {
                    logger_1.logger.error(setting_1.ErrorLevel.warn, undefined, protocol, 'Msock(race win sock) on error', err, targets);
                }
                : cbMap[ev]));
            logger_1.logger.log(setting_1.LogLevel.noisyDetail, msockPair.target, protocol, 'Race win and send cached datas to client');
            (_a = raceRecvDataMap.get(msockPair.sock)) === null || _a === void 0 ? void 0 : _a.forEach((d) => { var _a; return (_a = cbMap['data']) === null || _a === void 0 ? void 0 : _a.call(cbMap, d); });
            // raceRecvDataMap.clear();
        }
        if (win)
            winCb === null || winCb === void 0 ? void 0 : winCb();
        return win;
    };
    const sockMapper = (domainStats, i) => {
        const target = domainStats.target;
        let retryCount = 0;
        const raceRetry = () => __awaiter(void 0, void 0, void 0, function* () {
            yield new Promise((r) => setTimeout(r, setting_1.Settings.inSocketRetryDelay));
            retryCount++;
            if (msockPair || minRecvSockPair || retryCount > setting_1.Settings.inSocketMaxRetry || finished)
                return;
            socks[i] = createSock();
            return true;
        });
        const createSock = () => {
            const raceStartAt = Date.now();
            let dataStartAt = 0;
            let recvCount = 0;
            let lastTimeoutVerifyAt = 0;
            const sock = net_1.default.connect(target.port, target.ip, () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    logger_1.logger.log(setting_1.LogLevel.detail, target, protocol, 'Connect to target and call onLogicConnect');
                    yield protocol.onLogicConnect(target, sock);
                }
                catch (e) {
                    logger_1.logger.error(setting_1.ErrorLevel.debugDetail, target, protocol, 'Protocol connect failed:', e);
                    fail(yield raceRetry());
                    return;
                }
                connectedSocks.push({ sock, target });
                sock.on('data', onData);
                dataStartAt = Date.now();
                connectCb === null || connectCb === void 0 ? void 0 : connectCb();
                dataCache.forEach((d) => {
                    logger_1.logger.log(setting_1.LogLevel.noisyDetail, target, protocol, 'Write dataCache to target socket', d);
                    protocol.writeToTargetSock(d, sock, target);
                });
            }));
            const onData = (data) => {
                logger_1.logger.log(setting_1.LogLevel.detail, target, protocol, `On race data[${data.length}]:`, data, msockPair);
                recvCount++;
                maxRecvCount = Math.max(recvCount, maxRecvCount);
                if (msockPair) {
                    if (msockPair.sock !== sock && domainStats.status === 'good')
                        channelDiagnostic_1.notExactlyGoodStats.feedback(true, protocol.addr);
                    logger_1.logger.log(setting_1.LogLevel.noisyDetail, target, protocol, 'Race package will be drop because msockPair exist');
                    return;
                }
                const mCache = raceRecvDataMap.get(sock);
                if (mCache) {
                    mCache.push(data);
                    return;
                }
                const now = Date.now();
                const actionCost = now - dataStartAt;
                const raceCost = now - raceStartAt;
                let weightedCost = actionCost * setting_1.Settings.actionRaceCostRate + raceCost;
                const costBonused = target.notProxy && haveProxy ? setting_1.Settings.proxyCostBonus : 0;
                weightedCost += costBonused;
                if (weightedCost >= minRacingCost)
                    fail(new types_1.ErrorRaceFail('Error: race failed'));
                else {
                    minRacingCost = weightedCost;
                    minRecvSockPair = { sock, target };
                    raceRecvDataMap.set(sock, [data]);
                    minCancelCb === null || minCancelCb === void 0 ? void 0 : minCancelCb();
                    minCancelCb = () => fail(new types_1.ErrorRaceFail('Error: race failed becouse min cancelled'));
                    const winCb = () => sock.removeListener('data', onData);
                    if (!judgeWin(false, winCb) && judgeTimeOut === -Infinity) {
                        judgeTimeOut = Number(setTimeout(() => judgeWin(true, winCb), costBonused + actionCost));
                        if (target.notProxy && actionCost + raceCost < setting_1.Settings.goodSocketTimeout)
                            channelDiagnostic_1.notExactlyGoodStats.feedback(false, protocol.addr);
                        else if (targets.find((v) => v.status === 'good'))
                            channelDiagnostic_1.notExactlyGoodStats.feedback(true, protocol.addr);
                    }
                }
            };
            const fail = (errorOrRetry) => __awaiter(void 0, void 0, void 0, function* () {
                var _a;
                if (finished)
                    return;
                // If error type is ErrorIdleTimeout, use another request to verify network.
                if (errorOrRetry instanceof types_1.ErrorIdleTimeout &&
                    recvCount >= protocol.minCountShouldIdleTimeoutRecv) {
                    if (lastTimeoutVerifyAt > Date.now() - setting_1.Settings.socketIdleReverifyWaitMilli)
                        return;
                    lastTimeoutVerifyAt = Date.now();
                    if (yield protocol.doIdleTimeoutVerify(domainStats.target)) {
                        logger_1.logger.log(setting_1.LogLevel.notice, target, protocol, 'Idle verify success');
                        return;
                    }
                    else {
                        logger_1.logger.error(setting_1.ErrorLevel.debugDetail, target, protocol, 'Idle verify failed');
                    }
                }
                if (target.notProxy && domainStats.status === 'good') {
                    channelDiagnostic_1.notExactlyGoodStats.feedback(true, protocol.addr);
                }
                if ((minRecvSockPair === null || minRecvSockPair === void 0 ? void 0 : minRecvSockPair.sock) === sock) {
                    clearTimeout(judgeTimeOut);
                    judgeTimeOut = -Infinity;
                    minCancelCb = null;
                    minRecvSockPair = null;
                    minRacingCost = Infinity;
                }
                sock.destroy();
                socks[i] = null;
                connectedSocks = connectedSocks.filter((v) => v.sock !== sock);
                if ((msockPair === null || msockPair === void 0 ? void 0 : msockPair.sock) === sock || (errorOrRetry !== true && !socks.find((v) => v))) {
                    const error = errorOrRetry instanceof Error
                        ? errorOrRetry
                        : new Error('Race connect fail: all sock invalid');
                    logger_1.logger.error(setting_1.ErrorLevel.warn, target, protocol, 'All race connection failed', error, target, targets);
                    (_a = cbMap['error']) === null || _a === void 0 ? void 0 : _a.call(cbMap, error);
                    finished = true;
                    if (!targets.some((v) => v.target.notProxy))
                        channelDiagnostic_1.notExactlyGoodStats.clear(protocol.addr);
                }
            });
            sock.on('end', () => {
                var _a;
                if (sock === (msockPair === null || msockPair === void 0 ? void 0 : msockPair.sock)) {
                    (_a = cbMap['end']) === null || _a === void 0 ? void 0 : _a.call(cbMap);
                    finished = true;
                }
                else if (!recvCount)
                    raceRetry();
            });
            sock.on('error', fail);
            sock.setTimeout(setting_1.Settings.socketIdleTimeout, () => {
                fail(new types_1.ErrorIdleTimeout('Error: socket time out'));
            });
            return sock;
        };
        return createSock();
    };
    const socks = targets.map(sockMapper);
    /**
     * If proxies exist and only one taget passed，means the only target is good.
     * Here to do correction if the good target is not good exactly.
     */
    if (setting_1.Settings.proxys.length && targets.length === 1 && targets[0].target.notProxy) {
        setTimeout(() => {
            if (msockPair || finished)
                return;
            channelDiagnostic_1.notExactlyGoodStats.feedback(true, protocol.addr);
            const { port, dAndP } = targets[0];
            setting_1.Settings.proxys.forEach((target) => socks.push(sockMapper(new types_1.DomainChannelStats(protocol.addr, port, dAndP, target), socks.length)));
        }, setting_1.Settings.goodSocketTimeout);
    }
    setTimeout(() => {
        var _a;
        if (!msockPair) {
            socks.forEach((s) => s === null || s === void 0 ? void 0 : s.destroy());
            const error = new Error('Socket connect Timeout');
            logger_1.logger.error(setting_1.ErrorLevel.warn, undefined, protocol, error, targets);
            (_a = cbMap['error']) === null || _a === void 0 ? void 0 : _a.call(cbMap, error);
            finished = true;
        }
    }, setting_1.Settings.socketConnectTimeout);
    return {
        write(data) {
            if (!msockPair) {
                logger_1.logger.log(setting_1.LogLevel.noisyDetail, undefined, protocol, 'Write data to cache and connected race sockets', data, targets);
                dataCache.push(data);
                connectedSocks.forEach((s) => protocol.writeToTargetSock(data, s.sock, s.target));
            }
            else {
                logger_1.logger.log(setting_1.LogLevel.noisyDetail, msockPair.target, protocol, 'Write data to target socket', data);
                protocol.writeToTargetSock(data, msockPair.sock, msockPair.target);
            }
        },
        destroy() {
            var _a;
            logger_1.logger.log(setting_1.LogLevel.detail, undefined, protocol, 'RaceConnect Destory called', targets);
            finished = true;
            (_a = msockPair === null || msockPair === void 0 ? void 0 : msockPair.sock) === null || _a === void 0 ? void 0 : _a.destroy();
            socks.forEach((s) => s === null || s === void 0 ? void 0 : s.destroy());
        },
        on: (ev, cb) => {
            cbMap[ev] = cb;
        },
        getCurrentTarget: () => msockPair === null || msockPair === void 0 ? void 0 : msockPair.target,
    };
};
exports.raceConnect = raceConnect;
//# sourceMappingURL=raceConnect.js.map