"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDomainProxy = exports.overrideSetting = exports.Settings = exports.LogLevel = exports.ErrorLevel = exports.isDev = void 0;
exports.isDev = process.env.NODE_ENV === 'development';
var ErrorLevel;
(function (ErrorLevel) {
    ErrorLevel[ErrorLevel["off"] = 0] = "off";
    ErrorLevel[ErrorLevel["dangerous"] = 1] = "dangerous";
    ErrorLevel[ErrorLevel["important"] = 2] = "important";
    /** Errorlevel warn means may affect proxy usage, such as a main connection failed. */
    ErrorLevel[ErrorLevel["warn"] = 3] = "warn";
    /** Errorlevel debugDetail means info may helpful for debug, such as one race connection failed. */
    ErrorLevel[ErrorLevel["debugDetail"] = 4] = "debugDetail";
    ErrorLevel[ErrorLevel["notice"] = 5] = "notice";
    ErrorLevel[ErrorLevel["all"] = 10000] = "all";
})(ErrorLevel = exports.ErrorLevel || (exports.ErrorLevel = {}));
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["off"] = 0] = "off";
    LogLevel[LogLevel["important"] = 1] = "important";
    LogLevel[LogLevel["notice"] = 2] = "notice";
    LogLevel[LogLevel["detail"] = 3] = "detail";
    LogLevel[LogLevel["noisyDetail"] = 4] = "noisyDetail";
    LogLevel[LogLevel["all"] = 10000] = "all";
})(LogLevel = exports.LogLevel || (exports.LogLevel = {}));
exports.Settings = {
    socketConnectTimeout: 15000,
    socketIdleTimeout: 4000,
    socketIdleReverifyWaitMilli: 30000,
    goodSocketTimeout: 300,
    notExactlyGoodCountLimit: 3.5,
    inSocketMaxRetry: 3,
    inSocketRetryDelay: 300,
    proxys: [],
    port: 8008,
    host: '0.0.0.0',
    proxyCostBonus: 10,
    actionRaceCostRate: 3,
    maxPkgLossPct: 50,
    maxGoodLatency: 150,
    pingAsync: true,
    pingTimeout: 10,
    pingBatchCount: 50,
    dnsTimeout: 10000,
    cacheFile: './tunnel-proxy-cache.bin',
    /**
     * After how much hours of inactive period a cached domain should be judge to clean.
     *
     * Each time a domain is judged to clean, it's count will be devidied by 3, until 0,
     * which means this domain is to be deleted.
     */
    cacheDomainLife: 30 * 24,
    /** Set to true may help speeding up page switching when using Firefox */
    forceSeperateHttpRequest: false,
    /** time unit (millisecond) for the realTimeout methods */
    timeoutUnit: 500,
    useIpv6: false,
    errorLevel: exports.isDev ? ErrorLevel.warn : ErrorLevel.off,
    errorFilter: (target, protocol) => true,
    logLevel: exports.isDev ? LogLevel.important : LogLevel.off,
    logFilter: (target, protocol) => true,
    loggerTime: true,
    loggerFoldToLenLimit: 80,
    loggerInfoStringify: false,
};
const overrideSetting = (settings) => {
    Object.assign(exports.Settings, settings);
    exports.getDomainProxy = genGetDomainProxy();
};
exports.overrideSetting = overrideSetting;
// TODO: return targets,rather than first match target.
const genGetDomainProxy = () => {
    const domainTargetMap = new Map();
    const domainProxyPairs = [];
    exports.Settings.proxys.forEach((target) => {
        var _a;
        (_a = target.fixedDomains) === null || _a === void 0 ? void 0 : _a.forEach((s) => domainProxyPairs.push([s, target]));
    });
    return (domain) => {
        let target = domainTargetMap.get(domain);
        if (target)
            return target;
        domainProxyPairs.some((v) => {
            if (domain.match(v[0])) {
                target = v[1];
                domainTargetMap.set(domain, target);
                return true;
            }
        });
        return target;
    };
};
/**
 * Parse proxy-domain map from Settings and retrun a method for query.
 */
exports.getDomainProxy = genGetDomainProxy();
//# sourceMappingURL=setting.js.map