#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const setting_1 = require("./common/setting");
const commander_1 = require("commander");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const proxy_1 = require("./proxy");
const program = new commander_1.Command();
program
    .version(require('../package.json').version)
    .option('-p, --port <port number>', 'Port to listen on')
    .option('-h, --host <host>', 'Host to bind, default 0.0.0.0')
    .option('-c, --config <config file path>', 'use a config file')
    .option('-C, --cache <cache file path>', 'Use a cache file for statistics saving and restoring, ./tunnel-proxy-cache.bin by default')
    .parse(process.argv);
const exitWithError = (error) => {
    console.error(error);
    process.exit();
};
const opts = program.opts();
if (opts.port) {
    const port = Number(opts.port);
    if (Number.isInteger(port))
        setting_1.Settings.port = port;
    else
        exitWithError('Illegal port number');
}
if (opts.host)
    setting_1.Settings.host = opts.host;
if (opts.cache)
    setting_1.Settings.cacheFile = opts.cache;
if (opts.config) {
    const configPath = path_1.default.resolve(opts.config);
    if (!fs_1.default.existsSync(configPath))
        exitWithError('Config file not found');
    try {
        const config = require(configPath);
        Object.assign(setting_1.Settings, config);
    }
    catch (e) {
        exitWithError(`Illegal config file: ${e.message}`);
    }
}
proxy_1.startProxy();
if (setting_1.isDev) {
    try {
        require('../dev/settingOverride');
    }
    catch (e) {
        console.error(`Require "./dev/settingOverride" failed`, e);
    }
}
//# sourceMappingURL=main.js.map