"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startProxy = void 0;
const logger_1 = require("./common/logger");
const raceConnect_1 = require("./common/raceConnect");
const http_1 = require("./protocols/http");
const socks5_1 = require("./protocols/socks5");
const channelDiagnostic_1 = require("./stats/channelDiagnostic");
const net_1 = __importDefault(require("net"));
const setting_1 = require("./common/setting");
function sockConnect(targets, protocol) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!targets.length) {
            logger_1.logger.error(setting_1.ErrorLevel.dangerous, undefined, protocol, 'Not target passed to sockConnect');
            yield protocol.doFailFeedback();
            protocol.sock.destroy();
            return;
        }
        protocol.takeOver(targets);
    });
}
function startProxy() {
    channelDiagnostic_1.tryRestoreCache();
    let traceIdCount = 0;
    const sockIpHostSet = new Set();
    const server = new net_1.default.Server((sock) => {
        const traceId = logger_1.logger.doseLog() ? `${traceIdCount++}--${Date.now()}` : '';
        const strIpHost = `${sock.remoteAddress}:${sock.remotePort}`;
        if (sockIpHostSet.has(strIpHost)) {
            logger_1.logger.error(setting_1.ErrorLevel.dangerous, undefined, traceId, 'Origin socket rejected becouse ip-port pair exist');
            sock.destroy();
            return;
        }
        sockIpHostSet.add(strIpHost);
        sock.on('end', () => {
            sockIpHostSet.delete(strIpHost);
        });
        sock.once('data', (data) => __awaiter(this, void 0, void 0, function* () {
            logger_1.logger.log(setting_1.LogLevel.noisyDetail, undefined, traceId, 'On origin data.', data);
            let protocol;
            try {
                for (const Protocol of [socks5_1.ProtocolSocks5, http_1.ProtocolHttp]) {
                    protocol = yield new Protocol(sock, raceConnect_1.raceConnect, traceId).process(data);
                    if (protocol)
                        break;
                }
            }
            catch (e) {
                sock.destroy();
                logger_1.logger.error(setting_1.ErrorLevel.important, undefined, protocol, 'Error occured while processing first data: ', e);
                protocol = undefined;
            }
            if (!protocol) {
                // TODO: 返回错误信息, 可开关
                sock.destroy();
                return;
            }
            const { addr, port } = protocol;
            const targets = yield channelDiagnostic_1.getTargets(addr, port);
            logger_1.logger.log(setting_1.LogLevel.detail, undefined, protocol, 'New request', targets);
            sockConnect(targets, protocol);
        }));
    });
    server.listen(setting_1.Settings.port, setting_1.Settings.host);
    logger_1.logger.logVital(`Proxy listening on ${setting_1.Settings.host}:${setting_1.Settings.port}`);
    if (setting_1.isDev)
        logger_1.logger.logVital('Proxy is running in development mode');
    const doExit = () => __awaiter(this, void 0, void 0, function* () {
        logger_1.logger.logVital('Proxy exiting, waiting for cache saving ...');
        yield channelDiagnostic_1.trySaveCache();
        logger_1.logger.logVital('Cache saved, exist process ...');
        process.exit();
    });
    process.on('SIGINT', doExit);
    process.on('SIGTERM', doExit);
}
exports.startProxy = startProxy;
//# sourceMappingURL=proxy.js.map